/*
 * BGprocess.c
 *
 * Created: 08/07/2018 19:39:17
 *  Author: Miha
 */ 

// Reads the joystick, sets and draws the location of the player's paddle, stores joystick extremes to EEPROM


#include "BGprocess.h"
#include <avr/io.h>
#include "ADC.h"
#include "LCD_Ili9341.h"
#include "kbd.h"
#include "systime.h"
#include <stdlib.h>
#include <avr/eeprom.h>

void handleJoystick(struct Player *human, struct Joystick *joystick, uint16_t *ee_pointer){
	int valueY = ADC_ReadChannel(JOYSTICK_Y);
	static int oldx;
	int y = 0;
	
	if(valueY > joystick->maxY){
		joystick->maxY = valueY;
		eeprom_write_word(ee_pointer, joystick->maxY);
	}
	if(valueY < joystick->minY){
		joystick->minY = valueY;
		eeprom_write_word(ee_pointer+2, joystick->minY);
	}
	
	float percentY = (float)(valueY-joystick->minY)/(joystick->maxY-joystick->minY);

	// Move down
	if (percentY >= JOYSTICK_SENS){
		if (human->oldY >= FIELD_HEIGHT - PADDLE_HEIGHT) y = FIELD_HEIGHT - PADDLE_HEIGHT;
		else{
			y = human->oldY + PADDLE_SPEED_Y;
			UG_FillFrame(oldx, human->oldY, oldx + PADDLE_WIDTH, human->oldY + PADDLE_HEIGHT, C_BLACK);
		}
		UG_FillFrame(oldx, y, oldx + PADDLE_WIDTH, y + PADDLE_HEIGHT, C_RED);
		human->oldY = y;
	}
	
	// Move up
	//else if (valueY <= 512 - minDiff){
	else if (percentY <= 1-JOYSTICK_SENS){
		if (human->oldY <= TOP_CLEARANCE + TOP_MARGIN) {
			y = TOP_CLEARANCE;
			//UG_DrawLine(0, TOP_CLEARANCE, SCREEN_WIDTH, TOP_CLEARANCE, C_WHITE);

		}
		else{
			y = human->oldY - PADDLE_SPEED_Y;
			UG_FillFrame(oldx, human->oldY, oldx + PADDLE_WIDTH, human->oldY + PADDLE_HEIGHT, C_BLACK);
		}
		UG_FillFrame(oldx, y, oldx + PADDLE_WIDTH, y + PADDLE_HEIGHT, C_RED);
		human->oldY = y;
	}
	else{
		adjustDelay();
	}
	
	// Return current coordinates
	human->x = oldx;
	human->y = human->oldY;
}

// Sets, stores location and draws the AI paddle
void moveAIPaddle(struct Player *AI, struct Ball *ball){
	float randomizer = ((GetSysTick() % 50) + 10);
	if(ball->x > AI->skill*SCREEN_WIDTH * randomizer/100) {
		if ((ball->y) > (AI->y + PADDLE_HEIGHT/2) + GetSysTick() % 7){
			//move down
			if(AI->y > (FIELD_HEIGHT - PADDLE_HEIGHT)){
				AI->y = (FIELD_HEIGHT - PADDLE_HEIGHT);
			}
			else{
				AI->y = AI->oldY + PADDLE_SPEED_Y - GetSysTick() % 4;
			}
		}
		else if ((ball->y) < (AI->y) + GetSysTick() % 7){
			//move up
			AI->y = AI->oldY - PADDLE_SPEED_Y + GetSysTick() % 4;
		}
		UG_FillFrame(SCREEN_WIDTH - PADDLE_WIDTH, AI->oldY, SCREEN_WIDTH, AI->oldY + PADDLE_HEIGHT, C_BLACK);
		UG_FillFrame(SCREEN_WIDTH - PADDLE_WIDTH, AI->y , SCREEN_WIDTH, AI->y + PADDLE_HEIGHT, C_BLUE);
	}
	else{
		adjustDelay();
	}
	
	AI->oldY = AI->y;
}


// Moves the ball around in the game, calls checkCollision() and sets points if the player misses the ball, returns 1 if someone scored
int moveBall(struct Ball *ball, struct Player *human, struct Player *AI){
	static int oldx, oldy;
	static int counter;
	
	
	if(!(counter % BALL_DIVIDER)){
		if (ball->x <= PADDLE_WIDTH){
			int collisionHuman = checkCollision(ball, human);
			
			if (bounceOff(ball, collisionHuman)){
				//AI player gets points
				AI->score++;
				return 1;
			}
			
		}
		
		if(ball->x >= SCREEN_WIDTH - PADDLE_WIDTH - BALL_RADIUS){
			int collisionAI = checkCollision(ball, AI);
			
			if (bounceOff(ball, collisionAI)){
				// Human gets points
				human->score++;
				return 1;
			}
		}
		
		if(ball->y <= TOP_CLEARANCE + TOP_MARGIN){
			//bounce off
			UG_DrawLine(PADDLE_WIDTH, TOP_CLEARANCE, SCREEN_WIDTH - PADDLE_WIDTH, TOP_CLEARANCE, C_WHITE);
			ball->speed_y *= -1;
		}
		
		if(ball->y >= FIELD_HEIGHT - BALL_RADIUS){
			//bounce off
			ball->speed_y *= -1;
		}
		oldx = ball->x;
		oldy = ball->y;
		ball->x += ball->speed_x;
		ball->y += ball->speed_y;
		UG_FillFrame(oldx, oldy, oldx + BALL_RADIUS, oldy + BALL_RADIUS, C_BLACK);
		UG_FillFrame(ball->x, ball->y, ball->x + BALL_RADIUS, ball->y + BALL_RADIUS, C_WHITE);
		UG_FillFrame(human->oldX, human->oldY, human->oldX + PADDLE_WIDTH, human->oldY + PADDLE_HEIGHT, C_RED);
	}
	else adjustDelay();
	counter++;
	return 0;
}

// Checks for collision with paddles; returns 1 on collision, else 0
int checkCollision(struct Ball *ball, struct Player *player){
	if(ball->y + BALL_RADIUS > player->y - PADLLE_CORRECTION){
		if(ball->y  < player->y + PADDLE_HEIGHT/3) return 1;			// Bounce from upper third
		else if(ball->y < player->y + 2*PADDLE_HEIGHT/3) return	2;		// Bounce from middle third
		else if(ball->y < player->y + PADDLE_HEIGHT) return 3;			// Bounce from lower third
		else return 0;
	}
	
	else return 0; // No bounce
}

// Draws the ball at the beginning of the game and sets the coordinates
void drawBall(struct Ball *ball){
	srand(GetSysTick());
	float angle = (rand() % 10) / (float)100;

	ball->x = (SCREEN_WIDTH - BALL_RADIUS)/2;
	ball->y = (FIELD_HEIGHT - BALL_RADIUS)/2;
	UG_FillFrame(ball->x, ball->y, ball->x + BALL_RADIUS, ball->y + BALL_RADIUS, C_WHITE);

	if(rand()%2){
		ball->speed_x = 2 + BALL_SPEED_X * cos(angle);
	}
	else {
		ball->speed_x = -(2 + BALL_SPEED_X * cos(angle));
	}
	if(rand()%2){
		ball->speed_y = 2 + BALL_SPEED_Y * sin(angle);
	}
	else {
		ball->speed_y = -(2 + BALL_SPEED_Y * sin(angle));
	}

	if(abs(ball->speed_x)< ball->speed_y) ball->speed_x = 4;
}


// A hacky way of dealing with different speed of drawing, caused by the lag of the screen
void adjustDelay(){
	UG_FillFrame(SCREEN_WIDTH/3, FIELD_HEIGHT/3, SCREEN_WIDTH/3 + PADDLE_WIDTH, FIELD_HEIGHT/3 + PADDLE_HEIGHT, C_BLACK);
	UG_FillFrame(SCREEN_WIDTH/3, FIELD_HEIGHT/3, SCREEN_WIDTH/3 + PADDLE_WIDTH, FIELD_HEIGHT/3 + PADDLE_HEIGHT, C_BLACK);
}

// Prints the score to the screen and checks if a player won the game
int updateScore(struct Player *human, struct Player *AI){
	char humanScore[5];
	char AIscore[5];
	
	sprintf	(humanScore, "%d", human->score);
	UG_PutString(SCREEN_WIDTH/3, 0, humanScore);
	sprintf	(AIscore, "%d", AI->score);
	UG_PutString(2*SCREEN_WIDTH/3, 0, AIscore);
	
	if(human->score >= MAX_SCORE) return 1;
	else if (AI->score >= MAX_SCORE) return 2;
	else return 0;
}

// Resets the score and sets the parameters for a new game
void reset(struct Player *human, struct Player *AI, int *gameOver){
	AI->score = 0;
	human->score = 0;
	updateScore(human, AI);
	gameOver = 0;
}

// Resets the visual part of the game
void playfieldReset(struct Player *human, struct Player *AI, struct Ball *ball){
	UG_DrawLine(PADDLE_WIDTH, TOP_CLEARANCE, SCREEN_WIDTH - PADDLE_WIDTH, TOP_CLEARANCE, C_WHITE);
	UG_FillFrame(ball->x, ball->y, ball->x + BALL_RADIUS, ball->y + BALL_RADIUS, C_BLACK);
	UG_FillFrame(SCREEN_WIDTH - PADDLE_WIDTH, AI->y, SCREEN_WIDTH, AI->y + PADDLE_HEIGHT, C_BLUE);
	
	UG_FillFrame(human->oldX, 0, human->oldX + PADDLE_WIDTH, SCREEN_HEIGHT, C_BLACK);
	UG_FillFrame(AI->oldX, 0, AI->oldX + PADDLE_WIDTH, SCREEN_HEIGHT, C_BLACK);
	UG_FillFrame(END_GAME_SIGN_X, END_GAME_SIGN_Y, END_GAME_SIGN_X + 70, END_GAME_SIGN_Y + 20, C_BLACK);
	UG_FillFrame(PRESS_OK_X1, PRESS_OK_Y1, PRESS_OK_X2, PRESS_OK_Y2, C_BLACK);
	
	human->y = FIELD_HEIGHT/2;
	human->oldY= FIELD_HEIGHT/2;
	human->x = 0;
	human->oldX = 0;
	
	AI->y = FIELD_HEIGHT/2;
	AI->oldY = FIELD_HEIGHT/2;
	AI->x = SCREEN_WIDTH - PADDLE_WIDTH;
	AI->oldX = SCREEN_WIDTH - PADDLE_WIDTH;
	
	UG_FillFrame(human->x, human->y, human->x + PADDLE_WIDTH, human->y + PADDLE_HEIGHT, C_RED);
	UG_FillFrame(AI->x, AI->y, AI->x + PADDLE_WIDTH, AI->y + PADDLE_HEIGHT, C_BLUE);
}

// Shows the splash at the start of the game
void startSplash(struct Player *human, struct Player *AI, int *gameOver){
	UG_FillFrame(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, C_BLACK);
	UG_FontSelect(&RFONT_24X40);
	UG_PutString(SCREEN_WIDTH/2-90,SCREEN_HEIGHT/3, "P O N G");
	UG_FontSelect(&RFONT_8X12);
	UG_PutString(PRESS_OK_X1,PRESS_OK_Y1, "Press OK");
	
	while (1){
		KBD_Read();
		if (KBD_GetKey() == BTN_OK){
			UG_FillFrame(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, C_BLACK);
			reset(human, AI, gameOver);
			break;
		}
	}
}

// Determines how the ball bounces off the pad
int bounceOff(struct Ball *ball, int bounceType){
	if (bounceType == 1){
		ball->speed_x *= -1;
		ball->speed_y =  -(2 + GetSysTick() % 3);
		return 0;
	}
	else if (bounceType == 2){
		ball->speed_x *= -1;
		return 0;
	}
	else if (bounceType == 3){
		ball->speed_x *= -1;
		ball->speed_y =  (2 + GetSysTick() % 3);
		return 0;
	}
	else return 1;
}

// Calls all of the needed functions for the game to operate
int playGame(struct Player *human, struct Player *AI, struct Ball *ball, struct Joystick *joystick, uint16_t *ee_pointer){
	int move = moveBall(ball, human, AI);
	handleJoystick(human, joystick, ee_pointer);
	moveAIPaddle(AI, ball);
	return move;
}

// Shows the splash at the end of the game
void endSplash(int *gameOver){
	if (*gameOver == 1){
		UG_PutString(END_GAME_SIGN_X, END_GAME_SIGN_Y, "YOU WIN");
	}
	if (*gameOver == 2){
		UG_PutString(END_GAME_SIGN_X, END_GAME_SIGN_Y, "YOU LOSE");
	}
	_delay_ms(END_DELAY);
}

// Loads the calibration values of the ADC that handles the joystick from EEPROM
int loadJoystick(int maxOrMin, uint16_t *ee_pointer){
	if (maxOrMin) {
		return eeprom_read_word(ee_pointer+2);
	}
	else{
		return eeprom_read_word(ee_pointer);
	}
}

void init(){
	Systime_Init();
	sei();
	KBD_Init();
	LCD_Init();
	ADC_Init();
}


void waitForPlayer(struct Player *human, struct Player *AI, struct Ball *ball, int *gameOver){
	
	while (1){
		UG_PutString(PRESS_OK_X1,PRESS_OK_Y1, "Press OK");
		KBD_Read();
		if (KBD_GetKey() == BTN_OK){
			
			*gameOver = updateScore(human, AI);
			if(*gameOver){
				endSplash(gameOver);
				startSplash(human, AI, gameOver);
			}
			
			playfieldReset(human, AI, ball);
			drawBall(ball);
			break;
		}
	}
}