/*
 * BGprocess.h
 *
 * Created: 08/07/2018 19:35:34
 *  Author: Miha
 */ 


#ifndef BGPROCESS_H_
#define BGPROCESS_H_

#include <avr/io.h>


#define JOYSTICK_Y 2
#define JOYSTICK_X 3
#define SCREEN_WIDTH 320
#define SCREEN_HEIGHT 240
#define TOP_CLEARANCE 13
#define TOP_MARGIN 4
#define FIELD_HEIGHT 240
#define PADDLE_WIDTH 5
#define PADDLE_HEIGHT 30
#define PADDLE_SPEED_X 1
#define PADDLE_SPEED_Y 5
#define BALL_RADIUS 6
#define BALL_SPEED_X 4
#define BALL_SPEED_Y 4
#define BALL_DIVIDER 1	// Finetune the speed of the ball by skipping frames bigger number -> slower ball
#define GAME_DIVIDER 1000
#define MAX_SCORE 5
#define END_DELAY 1100
#define PADLLE_CORRECTION 2
#define JOYSTICK_SENS 0.75
#define END_GAME_SIGN_X (SCREEN_WIDTH/2 - 40)
#define END_GAME_SIGN_Y (SCREEN_HEIGHT/2)
#define PRESS_OK_X1 120
#define PRESS_OK_X2 200
#define PRESS_OK_Y1 200
#define PRESS_OK_Y2 210

struct Ball{
	int x;
	int y;
	int oldX;
	int oldY;
	int speed_x;
	int speed_y;
};

struct Player{
	int x;
	int y;
	int oldX;
	int oldY;
	int score;
	float skill; // Skill for AI is between 0 - 1 (excluding), lower number means better skill
};

struct Joystick{
	int maxY;
	int minY;
};




// Reads the joystick, sets and draws the location of the player's paddle
void handleJoystick(struct Player *human, struct Joystick *joystick, uint16_t *ee_pointer);
// Sets, stores location and draws the AI paddle
void moveAIPaddle();
// Moves the ball around in the game, calls checkCollision() and sets points if the player misses the ball, returns 1 if someone scored
int moveBall(struct Ball *ball, struct Player *human, struct Player *AI);
// Checks for collision with paddles; returns 1 on collision, else 0
int checkCollision(struct Ball *ball, struct Player *player);
// Draws the ball at the beginning of the game and sets the coordinates
void drawBall();
// A hacky way of dealing with different speed of drawing, caused by the lag of the screen
void adjustDelay();
// Prints the score to the screen and checks if a player won the game
int updateScore(struct Player *human, struct Player *AI);
// Resets the score and sets the parameters for a new game
void reset(struct Player *human, struct Player *AI, int *gameOver);
// Resets the visual part of the game
void playfieldReset(struct Player *human, struct Player *AI, struct Ball *ball);
// Determines how the ball bounces off the pad
int bounceOff(struct Ball *ball, int bounceType);
// Shows the splash at the start of the game
void startSplash();
// Shows the splash at the end of the game
void endSplash();
// Calls all of the needed functions for the game to operate
int playGame(struct Player *human, struct Player *AI, struct Ball *ball, struct Joystick *joystick, uint16_t *ee_pointer);
// Loads the calibration values of the ADC that handles the joystick from EEPROM
int loadJoystick(int maxOrMin, uint16_t* ee_pointer);
// Waits for the player after a point and resets the playfield
void waitForPlayer();
// Calls the initialization functions
void init();


#endif /* BGPROCESS_H_ */